@@ -55,17 +55,22 @@ def parse_cmdlineoptions(arglist):
 	try: optlist, args = getopt.getopt(arglist, "blr:sv:V",
 		 ["backup-mode", "calculate-average", "check-destination-dir",
 		  "compare", "compare-at-time=", "current-time=", "exclude=",
-		  "exclude-device-files", "exclude-filelist=",
-		  "exclude-filelist-stdin", "exclude-globbing-filelist=",
-		  "exclude-mirror=", "exclude-other-filesystems",
-		  "exclude-regexp=", "exclude-special-files", "force",
-		  "group-mapping-file=", "include=", "include-filelist=",
-		  "include-filelist-stdin", "include-globbing-filelist=",
-		  "include-regexp=", "list-at-time=", "list-changed-since=",
-		  "list-increments", "list-increment-sizes",
-		  "never-drop-acls", "no-acls", "no-compare-inode",
-		  "no-compression", "no-compression-regexp=", "no-eas",
-		  "no-file-statistics", "no-hard-links", "null-separator",
+		  "exclude-device-files", "exclude-fifos",
+		  "exclude-filelist=", "exclude-symbolic-links",
+		  "exclude-sockets", "exclude-filelist-stdin",
+		  "exclude-globbing-filelist=",
+		  "exclude-globbing-filelist-stdin", "exclude-mirror=",
+		  "exclude-other-filesystems", "exclude-regexp=",
+		  "exclude-special-files", "force", "group-mapping-file=",
+		  "include=", "include-filelist=", "include-filelist-stdin",
+		  "include-globbing-filelist=",
+		  "include-globbing-filelist-stdin", "include-regexp=",
+		  "include-special-files", "include-symbolic-links",
+		  "list-at-time=", "list-changed-since=", "list-increments",
+		  "list-increment-sizes", "never-drop-acls", "no-acls",
+		  "no-compare-inode", "no-compression",
+		  "no-compression-regexp=", "no-eas", "no-file-statistics",
+		  "no-hard-links", "null-separator",
 		  "override-chars-to-quote=", "parsable-output",
 		  "print-statistics", "remote-cmd=", "remote-schema=",
 		  "remove-older-than=", "restore-as-of=", "restrict=",
@@ -85,8 +90,15 @@ def parse_cmdlineoptions(arglist):
 			else: restore_timestr = arg
 		elif opt == "--current-time":
 			Globals.set_integer('current_time', arg)
-		elif opt == "--exclude": select_opts.append((opt, arg))
-		elif opt == "--exclude-device-files": select_opts.append((opt, arg))
+		elif (opt == "--exclude" or
+			  opt == "--exclude-device-files" or
+			  opt == "--exclude-fifos" or
+			  opt == "--exclude-other-filesystems" or
+			  opt == "--exclude-regexp" or
+			  opt == "--exclude-special-files" or
+			  opt == "--exclude-sockets" or
+			  opt == "--exclude-symbolic-links"):
+			select_opts.append((opt, arg))
 		elif opt == "--exclude-filelist":
 			select_opts.append((opt, arg))
 			select_files.append(sel_fl(arg))
@@ -96,12 +108,16 @@ def parse_cmdlineoptions(arglist):
 		elif opt == "--exclude-globbing-filelist":
 			select_opts.append((opt, arg))
 			select_files.append(sel_fl(arg))
-		elif (opt == "--exclude-other-filesystems" or
-			  opt == "--exclude-regexp" or
-			  opt == "--exclude-special-files"): select_opts.append((opt, arg))
+		elif opt == "--exclude-globbing-filelist-stdin":
+			select_opts.append(("--exclude-globbing-filelist",
+								"standard input"))
+			select_files.append(sys.stdin)
 		elif opt == "--force": force = 1
 		elif opt == "--group-mapping-file": group_mapping_filename = arg
-		elif opt == "--include": select_opts.append((opt, arg))
+		elif (opt == "--include" or
+			  opt == "--include-special-files" or
+			  opt == "--include-symbolic-links"):
+			select_opts.append((opt, arg))
 		elif opt == "--include-filelist":
 			select_opts.append((opt, arg))
 			select_files.append(sel_fl(arg))
@@ -111,6 +127,10 @@ def parse_cmdlineoptions(arglist):
 		elif opt == "--include-globbing-filelist":
 			select_opts.append((opt, arg))
 			select_files.append(sel_fl(arg))
+		elif opt == "--include-globbing-filelist-stdin":
+			select_opts.append(("--include-globbing-filelist",
+								"standard input"))
+			select_files.append(sys.stdin)
 		elif opt == "--include-regexp": select_opts.append((opt, arg))
 		elif opt == "--list-at-time":
 			restore_timestr, action = arg, "list-at-time"
@@ -178,7 +198,8 @@ def check_action():
 		if l == 2: pass # Will determine restore or backup later
 		else:
 			commandline_error("Switches missing or wrong number of arguments")
-	elif action == 'test-server': pass # test-server takes any number of args
+	elif action == 'test-server' or action == 'calculate-average':
+		pass # these two take any number of args
 	elif l > 2 or action not in arg_action_dict[l]:
 		commandline_error("Wrong number of arguments given.")
 
@@ -214,7 +235,7 @@ def init_user_group_mapping(destination_conn):
 		rp = rpath.RPath(Globals.local_connection, filename)
 		try: return rp.get_data()
 		except OSError, e:
-			log.FatalError("Error '%s' reading mapping file '%s'" %
+			Log.FatalError("Error '%s' reading mapping file '%s'" %
 						   (str(e), filename))
 	user_mapping_string = get_string_from_file(user_mapping_filename)
 	destination_conn.user_group.init_user_mapping(user_mapping_string)
@@ -272,6 +293,7 @@ def Backup(rpin, rpout):
 	init_user_group_mapping(rpout.conn)
 	backup_final_init(rpout)
 	backup_set_select(rpin)
+	backup_warn_if_infinite_regress(rpin, rpout)
 	if prevtime:
 		rpout.conn.Main.backup_touch_curmirror_local(rpin, rpout)
 		Time.setprevtime(prevtime)
@@ -311,7 +333,6 @@ def backup_check_dirs(rpin, rpout):
 		Log.FatalError("Source directory %s does not exist" % rpin.path)
 	elif not rpin.isdir():
 		Log.FatalError("Source %s is not a directory" % rpin.path)
-	backup_warn_if_infinite_regress(rpin, rpout)
 	Globals.rbdir = rpout.append_path("rdiff-backup-data")
 
 def backup_set_rbdir(rpin, rpout):
@@ -337,14 +358,16 @@ option.""" % rpout.path)
 
 def backup_warn_if_infinite_regress(rpin, rpout):
 	"""Warn user if destination area contained in source area"""
-	if rpout.conn is rpin.conn: # it's meaningful to compare paths
-		if ((len(rpout.path) > len(rpin.path)+1 and
-			 rpout.path[:len(rpin.path)] == rpin.path and
-			 rpout.path[len(rpin.path)] == '/') or
-			(rpin.path == "." and rpout.path[0] != '/' and
-			 rpout.path[:2] != '..')):
-			# Just a few heuristics, we don't have to get every case
-			if Globals.backup_reader.Globals.select_source.Select(rpout): Log(
+	# Just a few heuristics, we don't have to get every case
+	if rpout.conn is not rpin.conn: return
+	if len(rpout.path) <= len(rpin.path)+1: return
+	if rpout.path[:len(rpin.path)+1] != rpin.path + '/': return
+
+	relative_rpout_comps = tuple(rpout.path[len(rpin.path)+1:].split('/'))
+	relative_rpout = rpin.new_index(relative_rpout_comps)
+	if not Globals.select_mirror.Select(relative_rpout): return
+
+	Log(
 """Warning: The destination directory '%s' may be contained in the
 source directory '%s'.  This could cause an infinite regress.  You
 may need to use the --exclude option.""" % (rpout.path, rpin.path), 2)
@@ -402,7 +425,8 @@ def backup_set_fs_globals(rpin, rpout):
 		Log.FatalError("--never-drop-acls specified, but ACL support\n"
 					   "disabled on destination filesystem")
 
-	SetConnections.UpdateGlobal('preserve_hardlinks', dest_fsa.hardlinks)
+	if Globals.preserve_hardlinks != 0:
+		SetConnections.UpdateGlobal('preserve_hardlinks', dest_fsa.hardlinks)
 	SetConnections.UpdateGlobal('fsync_directories', dest_fsa.fsync_dirs)
 	SetConnections.UpdateGlobal('change_ownership', dest_fsa.ownership)
 	SetConnections.UpdateGlobal('chars_to_quote', dest_fsa.chars_to_quote)
@@ -489,7 +513,7 @@ def restore_set_fs_globals(target):
 		if src_support: Globals.rbdir.conn.Globals.set_local(conn_attr, 1)
 
 	target_fsa = target.conn.fs_abilities.get_fsabilities_readwrite(
-		'destination', target, 0)
+		'destination', target, 0, Globals.chars_to_quote)
 	Log(str(target_fsa), 3)
 	mirror_fsa = Globals.rbdir.conn.fs_abilities.get_fsabilities_restoresource(
 		Globals.rbdir)
@@ -508,7 +532,8 @@ def restore_set_fs_globals(target):
 		Log.FatalError("--never-drop-acls specified, but ACL support\n"
 					   "disabled on destination filesystem")
 
-	SetConnections.UpdateGlobal('preserve_hardlinks', target_fsa.hardlinks)
+	if Globals.preserve_hardlinks != 0:
+		SetConnections.UpdateGlobal('preserve_hardlinks', target_fsa.hardlinks)
 	SetConnections.UpdateGlobal('change_ownership', target_fsa.ownership)
 
 	if Globals.chars_to_quote is None: # otherwise already overridden
@@ -674,7 +699,8 @@ def single_set_fs_globals(rbdir):
 		SetConnections.UpdateGlobal(write_attr, 1)
 		rbdir.conn.Globals.set_local(conn_attr, 1)
 
-	fsa = rbdir.conn.fs_abilities.get_fsabilities_readwrite('archive', rbdir)
+	fsa = rbdir.conn.fs_abilities.get_fsabilities_readwrite('archive',
+								   rbdir, 1, Globals.chars_to_quote)
 	Log(str(fsa), 3)
 
 	update_triple(fsa.eas, ('eas_active', 'eas_write', 'eas_conn'))
@@ -685,7 +711,8 @@ def single_set_fs_globals(rbdir):
 	update_triple(fsa.carbonfile,
 				  ('carbonfile_active', 'carbonfile_write', 'carbonfile_conn'))
 
-	SetConnections.UpdateGlobal('preserve_hardlinks', fsa.hardlinks)
+	if Globals.preserve_hardlinks != 0:
+		SetConnections.UpdateGlobal('preserve_hardlinks', fsa.hardlinks)
 	SetConnections.UpdateGlobal('fsync_directories', fsa.fsync_dirs)
 	SetConnections.UpdateGlobal('change_ownership', fsa.ownership)
 	SetConnections.UpdateGlobal('chars_to_quote', fsa.chars_to_quote)
@@ -823,12 +850,12 @@ def checkdest_need_check(dest_rp):
 The rdiff-backup data directory
 %s
 exists, but we cannot find a valid current_mirror marker.  You can
-avoid this message by removing the rdiff_backup_data directory;
+avoid this message by removing the rdiff-backup-data directory;
 however any data in it will be lost.
 
 Probably this error was caused because the first rdiff-backup session
 into a new directory failed.  If this is the case it is safe to delete
-the rdiff_backup_data directory because there is no important
+the rdiff-backup-data directory because there is no important
 information in it.
 
 """ % (Globals.rbdir.path,))
