@@ -197,8 +197,7 @@ def cmp_attribs(rp1, rp2):
 	if Globals.change_ownership and rp1.getuidgid() != rp2.getuidgid():
 		result = None
 	elif rp1.getperms() != rp2.getperms(): result = None
-	# Don't compare ctime for now, add later
-	#elif rp1.getctime() != rp2.getctime(): result = None
+	elif rp1.getctime() != rp2.getctime(): result = None
 	elif rp1.issym() and rp2.issym(): # Don't check times for some types
 		result = 1
 	elif rp1.isblkdev() and rp2.isblkdev(): result = 1
@@ -840,15 +839,17 @@ class RPath(RORPath):
 		return uid == 0 or uid == self.data['uid']
 
 	def isgroup(self):
-		"""Return true if current process is in group of rp"""
-		return self.conn.Globals.get('process_gid') == self.data['gid']
+		"""Return true if process has group of rp"""
+		return self.data['gid'] in self.conn.Globals.get('process_groups')
 
 	def delete(self):
 		"""Delete file at self.path.  Recursively deletes directories."""
 		log.Log("Deleting %s" % self.path, 7)
 		if self.isdir():
 			try: self.rmdir()
-			except os.error: self.conn.shutil.rmtree(self.path)
+			except os.error:
+				if Globals.fsync_directories: self.fsync()
+				self.conn.shutil.rmtree(self.path)
 		else: self.conn.os.unlink(self.path)
 		self.setdata()
 
